# Pull Request (PR) reviews

Except for urgent or very small grammar or spelling fixes, such as simple
changes discussed below, we leave pull requests open for at least 24 hours, so
that others have the chance to review/comment.

### Favorable review

A favorable review is determined by the contents of the PR complying with the
contributing guide, the writing style, and agreement the contents align with the
TAG's goals, objectives, and scope.  It is anticipated that PRs submitted, with
the exception of spelling and grammar changes, have been discussed with members
of the TAG via slack or issues.

#### Nits

Nits are minor suggestions and changes that are strongly encouraged to be
identified and resolved to provide consistency in the repo.  Preferential
language or language that is a matter of preferred usage are not considered
nits.

##### Example of preferential language

> They use cloud technologies with clear understanding of risks and the ability
> to validate that their security policy decisions are reflected in deployed
> software.

"Ability" is a human oriented term, "capability" is more technical and may be
more appropriate.

Suggestion:
> They use cloud technologies with clear understanding of risks and the
> capability to validate their security policy decisions are reflected in
> deployed software.

##### Example of a nit

> They use cloud-native technologies with clear understanding of risks and the
> ability to validate that their security policy decisions are reflected in
> deployed software.

Per TOC definition of cloud native, it is not hyphenated.

correction:
> They use cloud native technologies...

#### Simple changes

Simple changes are defined as:

* spelling, typo, grammar
* clarifications, minor updates

A person without access, other than the PR author, can and _is_ encouraged to
review a PR and comment/+1 that they have done a review and found it favorable.
A person with access, including the PR author, may then perform the merge.

A person with access, other than the PR author, can both review **and** merge a
PR if found favorable after review.

[Code owners](/CODEOWNERS) need to be at least one concurring reviewer or the
merging party.

#### Significant changes

Significant changes are defined as:

* major changes to the repo
* extensive changes to repo contents
* other items as determined by the Technical Leads and Co-Chairs (to be updated
  here as they occur)

A person without access, other than the PR author can and _is_ encouraged to
review a PR and comment/+1 that they have done a review and found it favorable.
A second person with access, other than the PR Author, must also review the PR
and provide concurrence prior to merging.

Two persons with access, other than the PR author, must review the PR and
provide concurrence, the last of which should perform the merge.

[Code owners](/CODEOWNERS) need to be at least one concurring reviewer or the
merging party.

### Merging pull requests

PRs may be merged after at least one review as occurred, dependent on the type
of changes reflected in the PR.  The merging party needs to verify a review has
occurred, the PR is in alignment with this guide, and is in scope of the TAG.